@@ -12,6 +12,7 @@ our @EXPORT      = qw(REGION_INSIDE REGION_OUTSIDE
 		      match_regions
 		      classify_regions
 		      select_regions
+		      filter_regions
 		      merge_regions
 		      reverse_regions
 		      match_borders
@@ -181,29 +182,45 @@ sub select_regions {
 }
 
 sub select_regions_strict {
-    my @list = @{+shift};
-    my @by = @{+shift};
-    my $flag = shift;
+    my($list, $by, $flag) = @_;
+    my($inside, $overlap, $outside) = filter_regions($list, $by);
+    (($flag & REGION_AREA_MASK) == REGION_INSIDE) ? @$inside : @$outside;
+}
 
-    my(@outside, @inside);
+##
+## Split @input into @inside, @overlap, @outside by @filter and return
+## their pointers.
+##
+## 4th and 5th result is corresponding entry of @filter for @inside
+## and @overlap.
+##
+sub filter_regions {
+    my @input = @{+shift};
+    my @filter = @{+shift};
 
-    for (my $i = 0; $i < @by; $i++) {
-	my($from, $to) = @{$by[$i]};
-	while (@list and $list[0][0] < $from and $list[0][1] <= $from) {
-	    push @outside, shift(@list);
+    my(@inside, @overlap, @outside);
+    my(@inside_match, @overlap_match);
+
+    for (my $i = 0; $i < @filter; $i++) {
+	my($from, $to) = @{$filter[$i]};
+	while (@input and $input[0][0] < $from and $input[0][1] <= $from) {
+	    push @outside, shift @input;
 	}
-	while (@list and $list[0][0] < $from) {
-	    shift @list;
+	while (@input and $input[0][0] < $from) {
+	    push @overlap, shift @input;
+	    push @overlap_match, $filter[$i];
 	}
-	while (@list and $list[0][0] < $to and $list[0][1] <= $to) {
-	    push @inside, shift(@list);
+	while (@input and $input[0][0] < $to and $input[0][1] <= $to) {
+	    push @inside, shift @input;
+	    push @inside_match, $filter[$i];
 	}
-	while (@list and $list[0][0] < $to) {
-	    shift @list;
+	while (@input and $input[0][0] < $to) {
+	    push @overlap, shift @input;
+	    push @overlap_match, $filter[$i];
 	}
     }
-    push @outside, @list;
-    (($flag & REGION_AREA_MASK) == REGION_INSIDE) ? @inside : @outside;
+    push @outside, splice @input;
+    (\@inside, \@overlap, \@outside, \@inside_match, \@overlap_match);
 }
 
 sub merge_regions {
